package v004::RDF::Schema::Resource;
use strict;
use Carp;
use Data::Dumper;

use v004::RDF::Simple::Resource;
use v004::RDF::Schema::Model;

use vars qw( @ISA
	     );

@ISA = qw( v004::RDF::Simple::Resource );



sub label
{
    my( $self ) = @_;

    my $res = $self->get_objects( $self->model->rdfs('label') );

    return undef unless defined $res->[0];
    return $res->[0]->value;
}

sub desig
{
    my( $self ) = @_;

    my $label = $self->label;
    my $uri   = $self->uri;
    my $val   = "";
    if( $self->value )
    {
	$val = length( $self->value ) > 15 ? substr( $self->value, 0, 14)."..." : $self->value;
    }
    
    return $label ? "$label ($uri)" : $val ? $val : $uri;
}

sub get_domain_properties
{
    my( $self ) = @_;

    my %props = ();

    
    foreach my $class (@{ $self->get_objects( $self->model->rdf('type') ) })
    {
	foreach my $entry ( values %{ $class->get_domain_properties_for_class })
	{
	    # I should be returning the closest reference to the property
	    #
	    $props{ $entry->{'prop'}->uri } = $entry;
	}
    }

    my $resource = $self->model->rdfs('Resource');
    foreach(@{ $resource->get_subjects( $self->model->rdfs('domain') ) })
    {
	$props{ $_->uri } ||= { prop => $_, class => $resource };
    }
    
    my $property = $self->model->rdfs('Property');
    foreach my $sub (@{ $property->get_subjects( $self->model->rdf('type') ) })
    {
	next if $sub->has_property( $self->model->rdfs('domain') );
	$props{ $sub->uri } ||= { prop => $sub, class => $resource };
    }

    return \%props;
}

sub get_domain_properties_for_class
{
    my( $self ) = @_;
    my %properties = ();

    my $prop_list = $self->get_subjects( $self->model->rdfs('domain') );
    foreach( @$prop_list )
    {
	$properties{ $_->uri } ||= { prop => $_, class => $self };
    }

    my $subClassOf = $self->get_objects( $self->model->rdfs('subClassOf') );
    if( $#$subClassOf != -1 )
    {
	foreach my $class ( @$subClassOf )
	{
	    my $sub_properties = $class->get_domain_properties_for_class;
	    foreach my $entry ( values %$sub_properties )
	    {
		$properties{ $entry->{'prop'}->uri } ||= $entry;
	    }
	}
    }
    return \%properties;
}

sub is_instance_of
{
    my( $self, $object ) = @_;

    if( $object->uri eq $self->model->rdfs('Resource')->uri )
    {
	return not defined $self->value;
    }
    elsif( $object->uri eq $self->model->rdfs('Literal')->uri )
    {
	return defined $self->value;
    }
    elsif( $object->uri eq $self->model->rdfs('ContainerMembershipProperty')->uri )
    {
	my $nsrdf = $self->model->ns_rdf;
	return( $self->uri =~ /^$nsrdf\_\d+$/ ? 1 : undef );
    }

    my $type_list = $self->get_objects( $self->model->rdf('type') );

    foreach my $type ( @$type_list )
    {
	return 1 if $type->is_Class_or_subClassOf( $object );
    }

    return undef;
}

sub is_Class_or_subClassOf
{
    my( $self, $object ) = @_;

    return 1 if $self->uri eq $object->uri;

    my $class_list = $self->get_objects( $self->model->rdfs('subClassOf') );
    foreach my $class ( @$class_list )
    {
	return 1 if $class->is_Class_or_subClassOf( $object );
    }

    return undef;
}

1;








