package v004::RDF::Source::DBI::Model;
use strict;
use DBI;
use Carp;
use Data::Dumper;

sub init
{
    my( $self, $connect, $name, $passwd ) = @_;

    $connect or croak "Connection string missing";
    $name   ||= "";
    $passwd ||= "";
    my $dbi_options ||= 
    {
#	RaiseError => 1,
    };

    $self->{'cache'} = {}; #cache->{'node'}{'uri'} ( thing, method )

    $self->dbh( DBI->connect( $connect, $name, $passwd, $dbi_options ) );
}


sub DESTROY
{
    my( $self ) = @_;
    if( defined $self->dbh )
    {
	$self->dbh->disconnect;
    }
}

sub dbh
{
    my( $self, $conn ) = @_;
    $self->{'source'}{'dbh'} = $conn if defined $conn;
    return $self->{'source'}{'dbh'};
}


sub get_node_by_id
{
    my( $self, $id ) = @_;

    my $res = $self->new_node;
    $res->init_by_id( $id );
#    die Dumper $res;
    return $res;
}

sub get_arc_by_id
{
    my( $self, $id ) = @_;
    my $sta = $self->new_arc;
    $sta->init_by_id( $id );
    return $sta;
}


sub get_uri_by_id
{
    my( $self, $id ) = @_;

    my $sql = qq(select uri
		 from resources
		 where id=$id
		 );
    my( $name ) = $self->dbh->selectrow_array($sql);
    
    $name ||= $id;

    return $name;
}

sub get_id_by_uri
{
    my( $self, $res ) = @_;
    # Todo: implement aboutEachPrefix

    defined $res or confess "Undefined resource";

    my $sql = qq(select id 
		 from resources
		 where uri='$res'
		 and isprefix is null
		 );
    my( $id ) = $self->dbh->selectrow_array($sql);

    if( not defined $id )
    {
	my $ns_local = $self->ns_local;
	if( $res =~ /^$ns_local(\d+)$/ )
	{
	    $id = $1;
	}
	else
	{
	    confess "Can not find '$res' in this database";
	}
    }

    return $id;
}



#################################
#
#  Public methods
#
#################################

sub get_node_by_uri
{
    my( $self, $uri ) = @_;

    my $node = $self->{'cache'}{'node'}{$uri};
    unless( $node )
    {
	$node = $self->new_node;
	$node->init_by_uri( $uri );
	$self->{'cache'}{'node'}{$uri} = $node;
    }
    return $node;
}

sub get_arc_by_uri
{
    my( $self, $uri ) = @_;

    my $arc = $self->{'cache'}{'arc'}{$uri};
    unless( $arc )
    {
	$arc = $self->new_arc;
	$arc->init_by_uri( $uri );
	$self->{'cache'}{'arc'}{$uri} = $arc;
    }
    return $arc;
}

sub arc_exist
{
    my( $self, $pred, $subj, $obj ) = @_;

    my $pred_id = $pred->id;
    my $subj_id = $subj->id;
    my $obj_id  = $obj->id;

    my $sql = qq( select id
		  from statements
		  where pred = $pred_id
		  and subj = $subj_id
		  and obj = $obj_id
		  );
    my( $id ) = $self->dbh->selectrow_array( $sql );
    return 0 unless $id;

    return 1;
}

sub node_exist_with_uri
{
    my( $self, $uri ) = @_;
    my $euri = $self->dbh->quote( $uri );
    my $sql = qq(select id 
		 from resources 
		 where uri=$euri
		 );
    my( $id ) = $self->dbh->selectrow_array( $sql );
    if( not $id )
    {
	my $ns_local = $self->ns_local;
	$uri =~ /^$ns_local(\d+)$/ or return 0;
    }
    return 1;
}

sub add_node
{
    my( $self, $uri, $value, $lang ) = @_;

    if( $uri )
    {
	$self->node_exist_with_uri( $uri )
	    and die "$uri already exist";
    }
    my $id = $self->nextval('internal_id');

    if( $uri || $value || $lang )
    {
	$uri ||= $self->ns_local.$id;

	my $euri = $self->dbh->quote( $uri );
	my $evalue = $self->dbh->quote( $value );
	my $elang = $self->dbh->quote( $lang );

	my $sql = qq( insert into resources
		      ( id, uri, value, lang )
		      values 
		      ( $id, $euri, $evalue, $elang )
		      );
	$self->dbh->do( $sql ) 
	    or confess( "($sql) ".$self->dbh->errstr );
    }

    my $res = $self->new_node;
    $res->init_by_id( $id );
    return $res;
}

sub add_arc
{
    my( $self, $pred, $subj, $obj, $fact ) = @_;

#    warn Dumper \@_;
    my $pred_id = $pred->id;
    my $subj_id = $subj->id;
    my $obj_id  = $obj->id;
    if( defined $fact ) # differ between undef and 0
    {
	$fact = ( $fact ? 1 : 'null' );
    }
    else
    {
	$fact = 1;
    }

    if( $self->arc_exist( $pred, $subj, $obj ) )
    {
	die "Statement $pred_id, $subj_id, $obj_id already exist";
    }

    my $id = $self->nextval('internal_id');
    my $sql = qq( insert into statements
		  ( id, pred, subj, obj, fact )
		  values 
		  ( $id, $pred_id, $subj_id, $obj_id, $fact )
		  );
    $self->dbh->do( $sql ) 
	or confess("($sql)". $self->dbh->errstr);

    my $sta = $self->new_arc;
    $sta->init_by_id( $id );
    return $sta;
}

sub nextval
{
    my( $self, $seq ) = @_;

    my( $id ) = $self->dbh->selectrow_array( "select nextval('$seq')" );
    $id or die "Failed to get nextval";

    return $id;
}

sub get_ContainerMembershipProperty
{
    my( $self, $nr ) = @_;
    
    my $uri = $self->ns_rdf.'_'.$nr;
    my $prop = $self->get_node_by_uri( $uri );

    return $prop if $prop;

    return $self->add_node( $uri );
}

sub get_ContainerMembershipProperties
{
    my( $self ) = @_;

    my $nsrdf = $self->ns_rdf;

    my $sql = qq(select id, uri
		 from resources
		 where uri like '$nsrdf\\\\_%'
		 );
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;

    my @member_list = ();
    while( my( $id, $uri ) = $sth->fetchrow_array )
    {
	next unless $uri =~ /^$nsrdf\_(\d+)$/;
	push @member_list, $self->get_node_by_id( $id );
    }
    $sth->finish;
    return \@member_list;
}


sub get_Literals
{
    my( $self ) = @_;

    my $sql = qq(select id
		 from resources
		 where value is not null
		 );

    my @literals;
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;
    while( my( $id ) = $sth->fetchrow_array )
    {
	push @literals, $self->get_node_by_id( $id );
    }
    $sth->finish;
    return \@literals;
}

sub get_some_Resources
{
    my( $self ) = @_;

    my $sql = qq(select id
		 from resources
		 where value is null
		 );

    my @resources;
    my $sth = $self->dbh->prepare($sql);
    $sth->execute;
    while( my( $id ) = $sth->fetchrow_array )
    {
	push @resources, $self->get_node_by_id( $id );
    }
    $sth->finish;
    return \@resources;
}



sub find_statements
{
# find_statements(
# 		{
#		    with =>
# 	 	    {
#  			pred => '#type',
# 			obj  => '#Class',
# 		    },
# 		    subj_lacking =>
# 		    {
# 			pred => '#domain',
# 		    }
#		    
# 		}
# 		);
#
#
# This is the same for both "get all properties without domain"
# and "get all classes without subClassOf".
#
# select subj from statements 
# where pred=18 and obj=48 and 
# subj not in ( select subj 
# from statements where pred=49);
# 
}





1;




